;
true	equ	-1	; define true
false	equ	not true; define false
;
mx100	equ	not true; true if using EPSON MX-100 printer
micro84	equ	true	; true if using OKIDATA Microline u84
;
rdchr	equ	1	; read char from console
prnchr	equ	5	; write char to printer
prnt	equ	9	; print console buff	
const	equ	11	; check console stat
srchf	equ	17	; search first
srchn	equ	18	; search next
fcb	equ	5ch	; file control block
base	equ	0	; base address of the cp/m system
bdos	equ	5	; bdos entry address
;
	if	mx100
condon	equ	15	; condensed printing on
condoff	equ	18	; condensed printing off
spacing equ     65	; line spacing
forlen  equ	67	; sets form length
	endif
;
	if	micro84
condon	equ	28	; condensed printing on
condoff	equ	30	; condensed printing off
spacing equ     37	; line spacing
forlen  equ	70	; sets form length
	endif
;
lencol	equ	25	; sets length of column
;
bel	equ	7	; bell
lf	equ	10	; line feed
ff	equ	12	; form feed
cr	equ	13	; carriage return	
esc	equ	27	; escape
;
;
;
	org	100h
begin:	jmp	start
;
	if	mx100                        
;
prntinit:
;
	db  	esc,forlen,0,1          	; set page length at 1"
	db	esc,spacing,8,condon,'$'	; set line size to 8/72" + cond print on
;
resprint:	db	ff,condoff,esc,spacing,12,'$' 	; restore printer
;
	endif
;
	if	micro84                        
;
prntinit:
;
	db  	esc,forlen,2	          	; set page length at 1"
	db	esc,spacing,2,16,condon,'$'	; set line size to 8/72" + cond print on
;
resprint:	db	ff,condoff,esc,spacing,2,9,'$' 	; restore printer
;
	endif
;
;
start:
;
	lxi	h,0    	; save the old stack
	dad	sp	; [HL]=stack
	shld	stack	; save it
	lxi	sp,stack	;get new stack
;
; save file write request char
;
	lda	fcb+17
	sta	filesw
;
; no fcb specified?
;
	lxi	h,fcb+1
	mov	a,m
	cpi	' '
	jnz	gotfcb
;
; no fcb - make fcb all '?'
;
	mvi	b,11	; fn+ft count
;
qloop:
;
	mvi	m,'?'	; store '?' in fcb
	inx	h
	dcr  	b
	jnz	qloop
;
; look up the fcb in the directory
;
gotfcb:
;
	mvi	c,srchf	; search first
	lxi	d,fcb
	call	bdos	; read first
	inr	a	; were there any?
	sta	temp	; save
	jnz	initprnt	; got some - print title, continue
	lxi	h,nonmsg
	call	wrprnt
	jmp	exit
;
nonmsg	db	bel,'File not found$'
;
; set up printer commands
;
initprnt:
;
	lxi	h,prntinit	; print init codes for printer
	call	wrprnt
	lda	temp	; reload extent
	jmp	some
;
;
;
; read more directory entries		
;
moredir:
;
	mvi	c,srchn	; search next
	lxi	d,fcb
	call	bdos	; read dir entry
	inr	a	; check for end (0ffh)
	jz	sprint	; no more - sort and print
;
; point to directory entry 
;
some:
;
	dcr	a	; undo prev 'inr a'
	ani	3	; make modulus 4
	add	a	; multiply...
	add	a	; ..by 32 because
	add	a	; ..each directory
	add	a	; ..entry is 32
	add	a	; ..bytes long
	lxi	h,81h	; point to buffer (skip to fn/ft)
	add	l	; point to entry
	mov	l,a	; save (can't carry to [H])
;
; move entry to table
;
	xchg		; entry to [DE]
	lhld	nextt	; next table entry to [HL]
	mvi	b,31	; entry length
;
tmove:
;
	ldax	d	; get entry char
	mov	m,a	; store in table
	inx	d
	inx	h
	dcr	b	; more?
	jnz	tmove
	shld	nextt	; save updated table addr
	lda	count	; get prev count
	inr	a
	sta	count
	jmp	moredir
;
; sort and print
;
sprint:
;
	lda	count	; init the order table
	lxi	h,order
	lxi	d,table
	lxi	b,31	; entry length
;
bldord:
;
	mov	m,e	; save lo ord addr
	inx	h
	mov	m,d	; save hi ord addr
	inx	h
	xchg		; table addr to [HL]
	dad	b	; point to next entry
	xchg
	dcr	a	; more?
	jnz	bldord	; ..yes
	lda	count	; get count
	sta	scount	; save as number to sort
	dcr	a	; only 1 entry?
	jz	done	; ..yes, so skip sort
;
sort:
;
	xra	a	; get a zero
	sta	switch	; show none switched
	lda	scount	; get count
	dcr	a	; use 1 less
	sta	temp	; save number to compare
	sta	scount	; save highest entry
	jz	done	; exit if no more
	lxi	h,order	; point to order table
;
sortlp:
;
	call	compr	; compare 2 entries
	cm	swap	; swap if not in order
	inx	h	; bump order
	inx	h	; ..table pointer
	lda	temp	; get count
	dcr	a
	sta	temp
	jnz	sortlp	; continue
;
; one pass of sort done
;
	lda	switch	; any swaps done?
	ora	a
	jnz	sort
;
; sort is all done - print entries
;
done:
;
	lxi	h,dirhdg	; print heading
	call	wrprnt
	lxi	h,order
	shld	nextt
	mvi	a,3   
	sta	prcount
;
; print an entry
;
entry:
;
	mvi	c,const	; console status
	call	bdos	; any key pressed?
	dcr	a
	jz	abort	; yes, abort
	lhld	nextt	; get order table pointer
	mov	e,m	; get lo addr
	inx	h
	mov	d,m	; get hi addr
	inx	h
	shld	nextt	; save updated table pointer
	xchg		; table entry to [HL]
	mvi	b,8	; file name length
	call	printit	; type filename
	mvi	e,'.'
	call	print  
	mvi	b,3	; get the filetype
	call	printit
;
endext:
;
	lda	nfile	; get number files
	inr	a	; bump
	daa		; make decimal
	sta	nfile	; save it back
	lda     prcount
	dcr	a
	sta	prcount
	cpi	0   
	jz   	clcr
	lxi	h,spaces
  	call	wrprnt
	jmp	clcrplus
;
clcr:
;
	lxi     h,crlf         	; end, then print c/r
	call    wrprnt
	mvi	a,3
	sta	prcount
;
;see if more entries
;
clcrplus:
;
	lda	count
	dcr	a
	sta	count
	jnz	entry	; yes, more
	jmp	spfkey
;
;
; 
; compare routine for sort
;
compr:
;
	push	h	; save table addr
	mov	e,m	; load lo
	inx	h
	mov	d,m	; load hi
	inx	h
	mov	c,m
	inx	h
	mov	b,m
;
; bc, de now point to entries to be compared
;
	xchg
;
cmplp:
;
	ldax	b
	cmp	m
	inx	h
	inx	b
	jz	cmplp
	pop	h
	ret	; cond code tells all
;
;
;
; print character in [E]	
;
print:
;
	push	b
	push	d
	push	h
	mvi	c,prnchr
	call	bdos
	pop	h
	pop 	d
	pop	b
	ret
;
;
;
wrprnt:
;
	mvi	a,'$'	; end of message delimiter?  
	cmp	m
	rz 
	mov	e,m
	push	h
	call	print
	pop	h
	inx	h
	jmp	wrprnt
;
;
;
printit:
;
	mov	e,m
	call	print
	inx	h
	dcr	b
	jnz	printit
	ret
;
;
;
space:
;
	mvi	e,' '
	jmp	print
;
;
;
temp	ds	1	; save dir entry
;
;
;
; error exit
;
erxit:
;
	pop	d	; get msg
	mvi	c,prnt
	jmp	callb	; print msg, exit
;
;
;
; abort - read char entered
;
abort:
;
	mvi	c,rdchr
;
callb:
;
	call	bdos	; delete the char, and fall into exit
;
; exit - all done , restore stack
;
exit:
;
	lxi	h,resprint
	call 	wrprnt
	lhld	stack	; get old stack
	sphl	; move to stack
	ret	; ..and return
;
;
;
; routine to find and print character assignments of the function keys
;
spfkey:
;
	lxi	h,spfkhdg	; print heading
	call	wrprnt
	mvi	a,11	; set up to accept 11 addresses
	lhld	base+1	; get address of top of function key table
	lxi	d,07dh	; create offset to it
	dad	d	; we now point to last address
;
again:
;
	mov	d,m	; move lsb into d
	dcx	h
	mov	e,m	 	; move msb into e
	dcx	h	
	push	d	; stack for later
	dcr	a
 	jnz	again	; get another key
	pop	h	; save key 0 for last
	shld 	savaddr	; start addr for key 0
	pop	h
	shld	savlength	; end addr for key 0
	push	h   	; save start addr for next key
;
newkey:
;
	pop	h	; recover start address
	pop	b	; recover next address
	mov	a,c	; 
	sub	l	 	; determine length of command
	push	b	; save next address for later
 	sta	lengthflag	; store length of command
	push	h
	lda	entryflag	; determine column of print
	cpi	0	; 
	jnz	goprint  	; if col 1, print it
	lda	lengthflag	; if not compute adj length
	mov	c,a
	mov	b,a		                               
	mov	a,m
;
countchr:
;
	cpi	cr	; if the char is a cr  
	jnz	nocr	; add 3 to counter
	inr	c 	; because it will be printed
	inr	c	; as  <cr>  instead of one
	inr	c	; print position
;
nocr:
;
	cpi	1bh	; if the char is a control
	jc	countmor	; char add one for the print
	inr	c	; position for the ^.
;
countmor:
;
	inx	h
	dcr	b
	mov	a,m
	jnz	countchr
	mov	a,c
	cpi	lencol
	jc	goprint
	mvi	a,1
	sta	entryflag
	lxi	h,crlf
	call	wrprnt
;
goprint:
;
	lxi	h,adder
	inr	m
 	call	wrprnt
	lda	lengthflag
	pop	h
;
printmor:
;
	mov	e,m	; get a char to print
	push	psw		
	push	h
	mvi	a,cr	; if a cr then print <CR>
   	cmp	m
	jz	crchar
	mvi	a,1bh    	; if not, is it a control
	cmp	m	; char...if so print ^ in	
	jnc	conchar	; front of it.
	call	print	; if neither..just print it
;
crretrn:
;
	pop 	h
	pop	psw
	inx	h
  	dcr	a
   	jnz	printmor	; test for end of command
	push	psw
	lda	entryflag	; if the entry is in col 2
	cpi	0	; then print cr/lf first
	jz	lastentry
	mvi	a,0	; if not fix counter
	sta	entryflag
	lda	lengthflag
	cpi	lencol	; check to see if its too long
	jnc	lastentry	; if so then cr/lf
	lxi	h,col2	; if not print col delimiter
	call	wrprnt
	jmp	nuprint
;
lastentry:
;
	mvi	a,1	; fix column counter
	sta	entryflag
	lxi	h,crlf
	call	wrprnt	; print a cr/lf
;
nuprint:
;
	pop	psw
	lxi	h,flag1	; get ready to see if
	mov	a,m	; key 0 has been printed
	cpi	0	; if so....
	jz	devs	; now figure out hardware devices
	lxi	h,adder	; if not test for last key
	mov	a,m	
	cpi	'9'	; if not key #9, then
   	jnz	newkey	; go back and get another
	lxi	h,flag1	; get ready to print key 0
	dcr	m	; so set flag1 so we'll know
	lxi	h,adder	; also set key number
	mvi	m,02fh	; so that when its incremented
	lhld	savlength	; later it'll be a '0'
	push	h 	; then get the beg and end
 	lhld	savaddr	; addresses for key 0
	push	h	; and stack them....
	jmp 	newkey	; now go back and print it.
;
;
;
; figure out and display status of hardware devices
;
devs:
;
	lxi	h,devasgn	; heading for device assignments
	call	wrprnt
	lxi	h,prntyp; heading for printer type
	call	wrprnt
	lhld	base+1	; get bios warm boot address
	lxi	d,063h	; add offset to get to 'iobite' content
	dad	d
	mov	a,m	; get printer type byte
	push	h	; save the location
	cpi	040h	; serial printer?
	lxi	h,serprnt
	cz	wrprnt	; if so, tell'em its the serial printer
	cpi	080h	; parallel printer?
	lxi	h,parprnt
	cz	wrprnt	; if so, tell'em its the parallel printer
	cpi	0c0h	; IEEE-488 printer?
	lxi	h,prnt488
	cz	wrprnt	; tell'em its the IEEE-488 printer
	lxi	h,protcol	; heading for printer protocol
	call	wrprnt
	pop	h	; get the position
	inx	h	; onward to the printer protocol...
	mov	a,m	; get protocol type
	push	h	; save the location
	cpi	0	; hardware handshake?
	lxi	h,hwdprnt
	cz	wrprnt	; if so, tell'em hardware handshake
	cpi	1	; qume with ETX/ACK?
	lxi	h,etxack
	cz	wrprnt	; if so, tell'em software ETX/ACK handshake
	lxi	h,xonxoff	; diablo XON/XOFF?
	cz	wrprnt	; tell'em software XON/XOFF handshake
	lxi	h,autscrl	; heading for auto scroll type
	call	wrprnt
	pop	h	; get the position
	inx	h	; onward to the auto scroll status
	mov	a,m	; get auto scroll type
	push	h	; save the location
	cpi	0	; zero means auto scroll is off
	lxi	h,ascrlof
	cz	wrprnt	; if so, tell'em auto scroll is off
	cpi	0ffh	; all ones mean that auto scroll is on
	lxi	h,ascrlon
	cz	wrprnt	; if so, tell'em auto scroll is on
	lxi	h,baud	; heading for baud rate
	call	wrprnt
	pop	h	; get the position
	inx	h	; onward to the baud rate status
	mov	a,m	; get baud rate type
	cpi	055h	; 1200 baud?
	lxi	h,b1200
	cz	wrprnt	; if so, tell'em 1200 baud
	cpi	056h	; 300 baud?
	lxi	h,b300
	cz	wrprnt	; if not, tell'em 300 baud
	jmp	exit	; exit, all done
;
;
;
; subroutine to print a control character when it's needed
;
conchar:
;
	push	h
	mvi 	e,'^'
	call	print
	pop	h
	mov	a,m
	adi	40h
	mov	e,a
	call	print
	lxi	h,lengthflag
	inr	m
	jmp	crretrn
;
;
;
; subroutine to print <cr> when carriage return is encountered
;
crchar:
;
	lxi	h,crsymb
	call	wrprnt
	lxi	h,lengthflag
	inr	m
	inr	m
 	inr	m
	jmp	crretrn
;
;
;
; swap entries in the order table
;
swap:
;
	mvi	a,1
	sta	switch	; show a swap was made
	mov	c,m
	inx	h
	push	h	; save table addr+1
	mov	b,m
	inx	h
	mov	e,m
	mov	m,c
	inx	h
	mov	d,m
	mov	m,b
	pop	h
	mov	m,d
	dcx	h	; back pointer to correct loc'n
	mov	m,e
	ret
;
;
;
	ds	40	; 20 level stack area
stack	ds	2	; save old stack here
;
nfile	db	0	; number of files printed
;
nextt	dw	table	; next table entry
;
crsymb	db	'<cr>$'
;
col2	db	cr,'                              | $'
;
dirhdg	db	cr,lf,lf,'                  Disk Directory',cr,lf,lf,'$'
;
spfkhdg	db	cr,lf,lf,'               Special Function Keys',cr,lf,lf,'$'
;
devasgn	db	cr,lf,lf,'                 Device Asignments'
;
crlf	db	cr,lf,'$'
;
prntyp	db	cr,lf,'Printer type is $'
;
protcol	db	cr,lf,'Printer protocol is $'
;
autscrl	db	cr,lf,'Auto Scroll is $'
;
baud	db	cr,lf,'Baud Rate is $'
;
serprnt	db	'RS-232 Serial$'
;
parprnt	db	'Centronics Parallel$'
;
prnt488	db	'IEE-488 Parallel$'
;
hwdprnt	db	'Hardware RTS/DSR$'
;
etxack	db	'Software ETX/ACK$'
;
xonxoff	db	'Software XON/XOFF$'
;
ascrlon	db	'On$'
;
ascrlof	db	'Off$'
;
b1200	db	'1200$'
;
b300	db	'300',cr,lf,'$'
;
savaddr ds	2
;
savlength ds	2
;
lengthflag db	0
;
entryflag  db	1
;
flag1	db	1
;
adder	db	30h
;
spcs    db	' = $'
;
count	db	0	; entry count
;
scount	db	0	; number to sort
;
prcount db	0	; counter for items on line
;
spaces	db	'   |   $'
;
switch	db	0	; swap switch for sort
;
filesw	ds	1	; 'F' if writing file
;
bufad	dw	80h	; output addr
;
order	ds	128	; order table
;
table	equ	$	; read entries in here
;
;
;
	end	start
